/*
 Jonathan Berger
 Szymon Rusinkiewicz

 qsplat_gui_wx.cpp
 The wxWindows-specific class

 Copyright (c) 1999-2000 The Board of Trustees of the
 Leland Stanford Junior University.  All Rights Reserved.
 */

#include "wx/wx.h"
#include "wx/glcanvas.h"
#include "wx/image.h"

#include <sys/time.h>
#include <sys/unistd.h>
#include <sys/timeb.h>

#include "qsplat_gui_wx.h"
#include "qsplat_model.h"

#ifdef DARWIN
# include <Carbon/Carbon.h>
#endif

#define INIT_WINDOW_WIDTH 623
#define INIT_WINDOW_HEIGHT 400


//////////////////////////////////////////////////
//	QSplatWXGUI Implementation		//
//////////////////////////////////////////////////
//Prep the GUI
void QSplatWXGUI::Go()
{
    m_frame = new MainFrame(NULL, "QSplat", wxPoint(50, 50), wxSize(INIT_WINDOW_WIDTH, INIT_WINDOW_HEIGHT));
    m_frame->getCanvasDimensions(canvas_width, canvas_height);
    m_frame->Show(TRUE);
   
    wxMenuBar *menuBar = m_frame->GetMenuBar();
    if (!strncmp((const char *)glGetString(GL_RENDERER), "Mesa", 4)) {
        whichDriver = SOFTWARE_BEST;
        menuBar->Check(SOFTWARE_BEST, true);
    } else {
        whichDriver = OPENGL_POINTS;
        menuBar->Check(OPENGL_POINTS, true);
    }

    // Set the initial desiredrate, and update the slider
    if ((int)whichDriver < (int)SOFTWARE_GLDRAWPIXELS) {
        set_desiredrate(8.0f);
        m_frame->SetSlider(8.0f);
    } else {
        set_desiredrate(4.0f);
        m_frame->SetSlider(4.0f);
    }

    //Init menu
    menuBar->Check(ID_SHINY, true);
    menuBar->Check(ID_BACKFACE_CULL, true);
    menuBar->Check(ID_SHOW_REFINEMENT, true);
    menuBar->Check(ID_AUTO_SPIN, true);

    updatemenus();

    return;
}


// Update the menus to show what's available
void QSplatWXGUI::updatemenus()
{
    wxMenuBar *menuBar = m_frame->GetMenuBar();

    bool hardwareAvailable = ((int)whichDriver < (int)SOFTWARE_GLDRAWPIXELS);
    menuBar->Enable(ID_SHINY, hardwareAvailable);
    
    bool opengldriverAvailable = !(whichDriver == SOFTWARE ||
                            whichDriver == SOFTWARE_TILES ||
                            whichDriver == SOFTWARE_BEST);
    menuBar->Enable(ID_SHOW_LIGHT, opengldriverAvailable);
    menuBar->Enable(ID_SHOW_REFINEMENT, opengldriverAvailable);

    bool modelAvailable = (theQSplat_Model != NULL);
    menuBar->Enable(ID_RESET_VIEWER, modelAvailable);
    menuBar->Enable(ID_ABOUT_THIS_MODEL, modelAvailable);
}



void QSplatWXGUI::fullscreen(bool go_fullscreen)
{
    m_frame->ShowFullScreen(go_fullscreen);
}

void QSplatWXGUI::updaterate(const char *text)
{
    m_frame->SetStatusText(text, 1);
}
void QSplatWXGUI::updatestatus(const char *text)
{
    m_frame->SetStatusText(text, 2);
}

void QSplatWXGUI::aboutmodel()
{
    if (theQSplat_Model!=NULL) {
        display_dialog("About this Model", theQSplat_Model->comments.c_str());
    }
    else {
        display_dialog("", "No model loaded.");
    }
}

void QSplatWXGUI::aboutqsplat()
{
    char str[255];

    sprintf(str, "QSplat,  version %s\nby\nSzymon Rusinkiewicz\nGary King\n\nCopyright (c) 1999-2000 The Board of Trustees of the\nLeland Stanford Junior University. All Rights Reserved.", QSPLAT_VERSION);
    display_dialog("About QSplat", str);
    
}

void QSplatWXGUI::display_dialog(const char *title, const char *message)
{
    m_frame->DisplayDialog(title, message);
}

void QSplatWXGUI::swapbuffers()
{
    m_frame->m_canvas->SwapBuffers();
}

std::vector<EventRef> QSplatWXGUI::events_received;
bool QSplatWXGUI::abort_drawing(float time_elapsed)
{
    if (time_elapsed < 1.2f / rate() + 0.1f)
        return false;
    
    UInt32    eventClass;
    UInt32    eventKind;
    EventRef theEvent;
    while(ReceiveNextEvent(0, NULL, (EventTimeout)0, true, &theEvent) == noErr)
    {
        eventClass = GetEventClass(theEvent);
        eventKind  = GetEventKind(theEvent);

        events_received.push_back(theEvent);
        switch(eventClass) {
            case kEventClassKeyboard:
                return true;
            case kEventClassMouse:
                switch(eventKind) {
                    case kEventMouseDown:
                        return true;
                    case kEventMouseUp:
                        if (!dragging())
                            return true;
                        break;
                    case kEventMouseDragged:
                        if(!theQSplat_Model->coarsest())
                            return true;
                        else
                            break;
                }
                break;
        }
    }
    
    return false;

}

void QSplatWXGUI::end_drawing(bool bailed)
{
    QSplatGUI::end_drawing(bailed);

    while (!events_received.empty()) {
        EventRef theEvent = events_received.back();
        SendEventToEventTarget (theEvent, GetEventDispatcherTarget());
        ReleaseEvent(theEvent);
        events_received.pop_back();
    }

}
