/*
 port by Jonathan Berger
 Szymon Rusinkiewicz

 qsplat_gui_wxviewer.cpp
 MainFrame and GLCanvas

 Copyright (c) 1999-2000 The Board of Trustees of the
 Leland Stanford Junior University.  All Rights Reserved.
 */

#include "qsplat_gui_wxviewer.h"
#include "qsplat_gui.h"
#include "qsplat_gui_wx.h"

#define INIT_FRAMERATE 2.0
#define MIN_FRAMERATE 1
#define MAX_FRAMERATE 20

//////////////////////////////////////////////////
//	MainFrame Implementation		//
//////////////////////////////////////////////////
// Use the Driver enum for the Driver menu IDs
BEGIN_EVENT_TABLE(MainFrame, wxFrame)
EVT_MENU(wxID_EXIT, MainFrame::OnMenu)
EVT_MENU(ID_OPEN, MainFrame::OnMenu)
EVT_MENU(ID_SHINY, MainFrame::OnMenu)
EVT_MENU(ID_BACKFACE_CULL, MainFrame::OnMenu)
EVT_MENU(ID_SHOW_LIGHT, MainFrame::OnMenu)
EVT_MENU(ID_SHOW_REFINEMENT, MainFrame::OnMenu)
EVT_MENU(ID_AUTO_SPIN, MainFrame::OnMenu)
EVT_MENU(ID_GO_FULLSCREEN, MainFrame::OnMenu)
EVT_MENU(ID_RESET_VIEWER, MainFrame::OnMenu)
EVT_MENU(ID_SET_FRAMERATE, MainFrame::OnMenu)
EVT_MENU(ID_ABOUT_QSPLAT, MainFrame::OnMenu)
EVT_MENU(ID_ABOUT_THIS_MODEL, MainFrame::OnMenu)
EVT_MENU(OPENGL_POINTS, MainFrame::OnMenu)
EVT_MENU(OPENGL_POINTS_CIRC, MainFrame::OnMenu)
EVT_MENU(OPENGL_QUADS, MainFrame::OnMenu)
EVT_MENU(OPENGL_POLYS_CIRC, MainFrame::OnMenu)
EVT_MENU(OPENGL_POLYS_ELLIP, MainFrame::OnMenu)
EVT_MENU(SOFTWARE_GLDRAWPIXELS, MainFrame::OnMenu)
EVT_MENU(SOFTWARE, MainFrame::OnMenu)
EVT_MENU(SOFTWARE_TILES_GLDRAWPIXELS, MainFrame::OnMenu)
EVT_MENU(SOFTWARE_TILES, MainFrame::OnMenu)
EVT_MENU(SOFTWARE_BEST_GLDRAWPIXELS, MainFrame::OnMenu)
EVT_MENU(SOFTWARE_BEST, MainFrame::OnMenu)
EVT_MENU(OPENGL_POLYS_ELLIP_SMALL, MainFrame::OnMenu)
EVT_MENU(OPENGL_SPHERES, MainFrame::OnMenu)
EVT_SLIDER(ID_FRAME_SLIDER, MainFrame::OnSliderUpdate)
EVT_CLOSE(MainFrame::OnClose)
EVT_TIMER(ID_TIMER, MainFrame::OnTimer)
EVT_ACTIVATE( MainFrame::someOtherName )
END_EVENT_TABLE()

void MainFrame::OnClose(wxCloseEvent& WXUNUSED(event))
{
    Exit();
}

void MainFrame::someOtherName(wxActivateEvent& event)
{
    blit_active = event.GetActive();
    if(blit_active) GUI->need_redraw();
    OnActivate(event);
}

void MainFrame::OnTimer(wxTimerEvent& event)
{
    if (!GUI->do_redraw) {
        bool done = GUI->idle();
        // If the idle loop said that refinement was done,
        // wait for the next event
        if (!GUI->do_redraw && done) {
            return;
        }
    }

    // Do a redraw if necessary
    if (GUI->do_redraw) {
        GUI->redraw();
        GUI->do_redraw = false;
    }
}

// My frame constructor
MainFrame::MainFrame(wxFrame *frame, const wxString& title, const wxPoint& pos,
                     const wxSize& size, long style)
: wxFrame(frame, -1, title, pos, size, style)
{
    m_canvas = new GLCanvas(this);
    m_timer    = new wxTimer(this, ID_TIMER);
    //m_timer->Start(30);
    blit_active = true;
    
    initMenuBar();
    initStatusBar();
}

void MainFrame::DisplayDialog(const char *title, const char *message)
{
    wxMessageDialog messdlg(this, message, title, wxOK);
    messdlg.ShowModal();
}

void MainFrame::OnMenu(wxCommandEvent& event)
{
    wxMenuBar *menuBar = GetMenuBar();
    switch (event.GetId())
    {
        case ID_OPEN:
            Open();
            break;
        case ID_SHINY:
            GUI->set_shiny( menuBar->IsChecked(event.GetId()) );
            GUI->need_redraw();
            break;
        case ID_BACKFACE_CULL:
            GUI->set_backfacecull( menuBar->IsChecked(event.GetId()) );
            GUI->need_redraw();
            break;
        case ID_SHOW_LIGHT:
            GUI->set_showlight( menuBar->IsChecked(event.GetId()) );
            GUI->need_redraw();
            break;
        case ID_SHOW_REFINEMENT:
            GUI->set_showprogressbar( menuBar->IsChecked(event.GetId()) );
            GUI->need_redraw();
            break;
        case ID_AUTO_SPIN:
            GUI->set_touristmode( !menuBar->IsChecked(event.GetId()) ); //note the exclamation point
            GUI->need_redraw();
            break;
        case ID_GO_FULLSCREEN:
            GUI->fullscreen( menuBar->IsChecked(event.GetId()) );
            break;
        case ID_RESET_VIEWER:
            GUI->resetviewer();
            GUI->need_redraw();
            break;
        case ID_SET_FRAMERATE:
            break;
        case ID_ABOUT_QSPLAT:
            GUI->aboutqsplat();
            break;
        case ID_ABOUT_THIS_MODEL:
            GUI->aboutmodel();
            break;
        case OPENGL_POINTS:
        case OPENGL_POINTS_CIRC:
        case OPENGL_QUADS:
        case OPENGL_POLYS_CIRC:
        case OPENGL_POLYS_ELLIP:
        case SOFTWARE_GLDRAWPIXELS:
        case SOFTWARE:
        case SOFTWARE_TILES_GLDRAWPIXELS:
        case SOFTWARE_TILES:
        case SOFTWARE_BEST_GLDRAWPIXELS:
        case SOFTWARE_BEST:
        case OPENGL_POLYS_ELLIP_SMALL:
        case OPENGL_SPHERES:
            menuBar->Check(GUI->whichDriver, false);
            GUI->whichDriver = (Driver) event.GetId();
            GUI->updatemenus();
            GUI->resetviewer(true);
            GUI->need_redraw();
            break;
        case wxID_EXIT:
            Exit();
            break;
    }

}

void MainFrame::Exit()
{
    m_timer->Stop();
    Destroy();
}

void MainFrame::initStatusBar()
{
    wxStatusBar *statusbar = CreateStatusBar(3);
    if(statusbar != NULL) {
        wxPanel *panel = new wxPanel(statusbar, -1, wxPoint(0, 1), wxSize(190, 20));
        m_frame_slider = new wxSlider(panel, ID_FRAME_SLIDER,
                                      (int) (2 * INIT_FRAMERATE), 2 * MIN_FRAMERATE, 2 * MAX_FRAMERATE,
                                      wxPoint(1, 0), wxSize(130, 20));
        wxString init_framerate;
        init_framerate << INIT_FRAMERATE;
        m_frame_text = new wxStaticText(panel, ID_FRAME_TEXT,
                                        "",
                                        wxPoint(140,0), wxSize(50, 20), wxST_NO_AUTORESIZE);
        setSliderLabel((float) INIT_FRAMERATE);
    }    
}

void MainFrame::initMenuBar()
{
    // Make a menubar
    wxMenuBar *menuBar = new wxMenuBar;
    wxMenu *file, *options, *driver, *commands, *help;
    wxMenu *opengl_sub, *software_sub, *visual_sub;

    //File
    file = new wxMenu;
    file->Append(ID_OPEN, "&Open\tCtrl-O" );
    // Quit now placed in Application menu for Mac OS X
#ifndef __DARWIN__
    file->Append(wxID_EXIT, "&Quit");
#endif
    menuBar->Append(file, "&File");

    //Options
    options = new wxMenu;
    options->Append(ID_SHINY, "&Shiny\tCtrl-S", "", true); //fouth arguement is "checkable"
    options->Append(ID_BACKFACE_CULL, "&Backface Cull\tCtrl-B", "", true);
    options->Append(ID_SHOW_LIGHT, "&Show Light", "", true);
    options->Append(ID_SHOW_REFINEMENT, "&Show Refinement", "", true);
    options->Append(ID_AUTO_SPIN, "&Auto-spin", "", true);
    menuBar->Append(options, "&Options");

    driver = new wxMenu;
    opengl_sub = new wxMenu;
    driver->Append(ID_OPENGL_SUB, "&OpenGL", opengl_sub);
    opengl_sub->Append(OPENGL_POINTS, "&Points", "", true);
    opengl_sub->Append(OPENGL_POINTS_CIRC, "&Round Points", "", true);
    opengl_sub->Append(OPENGL_QUADS, "&Quads", "", true);
    opengl_sub->Append(OPENGL_POLYS_CIRC, "&Poly Circles", "", true);
    opengl_sub->Append(OPENGL_POLYS_ELLIP, "&Poly Ellipses", "", true);
    software_sub = new wxMenu;
    driver->Append(ID_SOFTWARE_SUB, "&Software", software_sub);
    software_sub->Append(SOFTWARE_GLDRAWPIXELS, "&Z-Buffer + GL Blit", "", true);
    software_sub->Append(SOFTWARE, "&Z-Buffer", "", true);
    software_sub->Append(SOFTWARE_TILES_GLDRAWPIXELS, "&Tiles + GL Blit", "", true);
#ifndef WX
    // qsplat_draw_software_tiles is not yet implemented for wxWindows
    software_sub->Append(SOFTWARE_TILES, "&Tiles", "", true);
#endif
    software_sub->Append(SOFTWARE_BEST_GLDRAWPIXELS, "&Optimal + GL Blit", "", true);
    software_sub->Append(SOFTWARE_BEST, "&Optimal", "", true);
    visual_sub = new wxMenu;
    driver->Append(ID_VISUALIZATIONS_SUB, "&Visualizations", visual_sub);
    visual_sub->Append(OPENGL_POLYS_ELLIP_SMALL, "&Small Ellipses", "", true);
    visual_sub->Append(OPENGL_SPHERES, "&Spheres", "", true);
    menuBar->Append(driver, "&Driver");

    //Commands
    commands = new wxMenu;
#ifndef __DARWIN__
    commands->Append(ID_GO_FULLSCREEN, "&Go Fullscreen" );
#endif
    commands->Append(ID_RESET_VIEWER, "&Reset Viewer" );
#ifndef __DARWIN__
    commands->Append(ID_SET_FRAMERATE, "&Set Framerate" );
#endif
    menuBar->Append(commands, "&Commands");

    //Help
    help = new wxMenu;
    help->Append(ID_ABOUT_QSPLAT, "&About QSplat" );
    help->Append(ID_ABOUT_THIS_MODEL, "&About this model" );
    menuBar->Append(help, "&Help "); //

    SetMenuBar(menuBar);
}

void MainFrame::Open()
{
    wxFileDialog filedlg(this,
                         _("Choose a file to open"),
                         _(""),
                         _(""),
                         _("Models (*.qs)|*.qs|All files|*.*"),
                         wxOPEN | wxFILE_MUST_EXIST);

    if (filedlg.ShowModal() == wxID_OK)
    {
        GUI->OpenModel(filedlg.GetPath().c_str());
    }

}

void MainFrame::OnSliderUpdate(wxScrollEvent &event)
{

    float fps = (float) event.GetPosition() / (float) 2;
    setSliderLabel(fps);
    GUI->set_desiredrate(fps);
    GUI->need_redraw();
}

void MainFrame::OnChar(wxKeyEvent& event)
{
    wxMenuBar *menuBar = GetMenuBar();
    switch ( event.KeyCode() )
    {
        case 'o':
            if(event.m_metaDown)
            {
                Open();
            }
            break;
        case 'b':
            menuBar->Check(ID_BACKFACE_CULL, !menuBar->IsChecked(ID_BACKFACE_CULL));
            GUI->set_backfacecull(menuBar->IsChecked(ID_BACKFACE_CULL));
            GUI->need_redraw();
            break;
        case 'd':
        case 'f':
            menuBar->Check(ID_GO_FULLSCREEN, !menuBar->IsChecked(ID_GO_FULLSCREEN));
            GUI->fullscreen(menuBar->IsChecked(ID_GO_FULLSCREEN));
            GUI->need_redraw();
            break;
        case 's':
            menuBar->Check(ID_SHINY, !menuBar->IsChecked(ID_SHINY));
            GUI->set_shiny(menuBar->IsChecked(ID_SHINY));
            GUI->need_redraw();
            break;
        case ' ':
            GUI->resetviewer();
            GUI->need_redraw();
            break;
    }
}

void MainFrame::getCanvasDimensions(int &width, int &height)
{
    m_canvas->GetClientSize(&width, &height);
}

// This method sets the GUI slider to val
void MainFrame::SetSlider(float val)
{
    m_frame_slider->SetValue((int) (2 * val));
    setSliderLabel(val);
}

void MainFrame::setSliderLabel(float val)
{
    wxString s;
    s.Printf("%.1f fps", val);
    m_frame_text->SetLabel(s);    
}




/*----------------------------------------------------------------
Implementation of GLCanvas
-----------------------------------------------------------------*/

BEGIN_EVENT_TABLE(GLCanvas, wxGLCanvas)
EVT_SIZE(GLCanvas::OnSize)
EVT_PAINT(GLCanvas::OnPaint)
EVT_ERASE_BACKGROUND(GLCanvas::OnEraseBackground)
EVT_ENTER_WINDOW( GLCanvas::OnEnterWindow )
EVT_LEFT_DOWN(GLCanvas::OnMouseEvent)
EVT_LEFT_UP(GLCanvas::OnMouseEvent)
EVT_MIDDLE_DOWN(GLCanvas::OnMouseEvent)
EVT_MIDDLE_UP(GLCanvas::OnMouseEvent)
EVT_RIGHT_DOWN(GLCanvas::OnMouseEvent)
EVT_RIGHT_UP(GLCanvas::OnMouseEvent)
EVT_MOTION(GLCanvas::OnMouseEvent)
EVT_CHAR(GLCanvas::OnChar)
END_EVENT_TABLE()

//EVT_MOUSE_EVENTS( GLCanvas::OnMouseEvent ) //this caused extra strange events


//necessary because the top wxWindows objects captures all events
void GLCanvas::OnChar(wxKeyEvent& event)
{
    GUI->m_frame->OnChar(event);
}





void GLCanvas::OnEnterWindow( wxMouseEvent& event )
{
    SetFocus();
}

void GLCanvas::OnPaint( wxPaintEvent& event )
{
    if (!m_init)
    {
        SetCurrent();
        m_init = TRUE;
    }
    GUI->redraw();
}

void GLCanvas::OnSize(wxSizeEvent& event)
{
    // this is also necessary to update the context on some platforms
wxGLCanvas::OnSize(event);

    // set GL viewport (not called by wxGLCanvas::OnSize on all platforms...)
    int w,h;
    GetClientSize(&w, &h);
    if(GUI != NULL) {
        GUI->canvas_width = w;
        GUI->canvas_height = h;
        GUI->resetviewer(true);
        GUI->need_redraw();
    }
#ifndef __WXMOTIF__
    if (GetContext())
#endif
    {
        SetCurrent();
        glViewport(0, 0, (GLint) w, (GLint) h);
    }

}

void GLCanvas::OnEraseBackground(wxEraseEvent& event)
{
    // Do nothing, to avoid flashing.
}


// Only intercepts mouse motions events occuring over GLCanvas
void GLCanvas::OnMouseEvent(wxMouseEvent& event)
{
    int x = (int) event.GetX();
    int y = GUI->canvas_height - (int) event.GetY();

    if((!event.LeftIsDown() || event.LeftUp()) && (!event.RightIsDown())) {
        GUI->mouse(x,y,QSplatGUI::NO_BUTTON);
    }
    else if((event.LeftIsDown() && event.RightIsDown()) ||
            (event.RightIsDown() && event.ShiftDown()))
        GUI->mouse(x,y,QSplatGUI::TRANSZ_BUTTON);
    else if(event.LeftIsDown() && event.ShiftDown())
        GUI->mouse(x,y,QSplatGUI::LIGHT_BUTTON);
    else if(event.RightIsDown())
        GUI->mouse(x,y,QSplatGUI::TRANSXY_BUTTON);
    else if(event.LeftIsDown() && !event.LeftUp()) {
        GUI->mouse(x,y,QSplatGUI::ROT_BUTTON);
    }

}


